import pytest

from .. import Store, SettingNotFoundException


class DictBackend:
    def __init__(self, **settings):
        self._settings = settings

    def get_setting(self, key):
        return self._settings.get(key)


def test_store_init():
    Store([])
    Store([DictBackend()])
    # This error is not caught at the moment
    Store([DictBackend])

    with pytest.raises(Exception):
        Store()
    with pytest.raises(Exception):
        Store(None)


def test_store_get_setting():
    store = Store([DictBackend(key='secrets!')])

    value = store.get_setting('key')

    assert value == 'secrets!'


def test_store_get_setting_with_default():
    store = Store([DictBackend(key='secrets!')])

    value = store.get_setting('key', 'default')

    assert value == 'secrets!'


def test_store_get_setting_missing():
    store = Store([])

    with pytest.raises(SettingNotFoundException):
        store.get_setting('key')


def test_store_get_setting_missing_with_default():
    store = Store([DictBackend()])

    value = store.get_setting('key', 'default value')

    assert value == 'default value'


def test_store_add_backend():
    store = Store([])

    store.add_backend(DictBackend(environment='staging'))

    assert store.get_setting('environment') == 'staging'



import io
import sys

import pretend
import pytest

from ..backends.dotenv import DotenvBackend


if sys.version_info[0] == 3:
    builtins_open = 'builtins.open'
else:  # pragma: no cover
    builtins_open = '__builtin__.open'


def test_dotenv_init_bad_install(monkeypatch):
    monkeypatch.setattr('configstore.backends.dotenv.dotenv', None)

    with pytest.raises(ImportError):
        DotenvBackend('/secrets/app')


def test_dotenv_init_bad_file(monkeypatch):
    with pytest.raises(IOError):
        DotenvBackend('/this/does/not/exist')

    with pytest.raises(IOError):
        DotenvBackend('/home')


def test_dotenv_success(monkeypatch):
    content = u'APP_SECRET_KEY=1234abcd'
    fake_open = pretend.call_recorder(lambda path: io.StringIO(content))
    monkeypatch.setattr(builtins_open, fake_open)

    b = DotenvBackend('/some/path')
    value = b.get_setting('APP_SECRET_KEY')

    assert value == '1234abcd'
    assert fake_open.calls == [pretend.call('/some/path')]


def test_dotenv_missing(monkeypatch):
    content = u'APP_SECRET_KEY=1234abcd'
    fake_open = lambda path: io.StringIO(content)
    monkeypatch.setattr(builtins_open, fake_open)

    b = DotenvBackend('/some/path')
    value = b.get_setting('APP_DEBUG')

    assert value is None



try:
    import boto3
    from botocore.exceptions import ClientError
except ImportError:  # pragma: no cover
    boto3 = None


class AwsSsmBackend(object):
    """Backend for AWS System Manager Parameter Store.

    You can create an instance with a prefix:

        AwsSsmBackend('/myapp/pre-prod/')

    so that a call like store.get_setting('DEBUG') will
    try to get a parameter named `/myapp/pre-prod/DEBUG`.
    """

    def __init__(self, name_prefix=''):
        if boto3 is None:
            raise ImportError('install configstore[awsssm] to use '
                              'the AWS SSM backend')

        self.name_prefix = name_prefix

    def get_setting(self, param):
        client = boto3.client('ssm')
        name = self.name_prefix + param
        try:
            res = client.get_parameter(Name=name, WithDecryption=True)
        except ClientError as exc:
            if exc.response['Error']['Code'] == 'ParameterNotFound':
                return None
            else:
                raise

        return res['Parameter']['Value']



import pretend
import pytest

from botocore.exceptions import ClientError

from ..backends.awsssm import AwsSsmBackend


def test_awsssm_init_bad_install(monkeypatch):
    monkeypatch.setattr('configstore.backends.awsssm.boto3', None)

    with pytest.raises(ImportError):
        AwsSsmBackend()


def test_awsssm_success(monkeypatch):
    response = {'Parameter': {
        'Value': 'postgres://localhost/app',
    }}
    fake_client = pretend.stub(
        get_parameter=pretend.call_recorder(lambda Name, WithDecryption: response),
    )
    fake_boto3 = pretend.stub(
        client=pretend.call_recorder(lambda service: fake_client),
    )
    monkeypatch.setattr('configstore.backends.awsssm.boto3', fake_boto3)

    b = AwsSsmBackend()
    value = b.get_setting('DATABASE_URL')

    assert value == 'postgres://localhost/app'
    assert fake_boto3.client.calls == [pretend.call('ssm')]
    assert fake_client.get_parameter.calls == [
        pretend.call(Name='DATABASE_URL', WithDecryption=True),
    ]


def test_awsssm_success_with_prefix(monkeypatch):
    response = {'Parameter': {
        'Value': 'off',
    }}
    fake_client = pretend.stub(
        get_parameter=pretend.call_recorder(lambda Name, WithDecryption: response),
    )
    fake_boto3 = pretend.stub(
        client=pretend.call_recorder(lambda service: fake_client),
    )
    monkeypatch.setattr('configstore.backends.awsssm.boto3', fake_boto3)

    b = AwsSsmBackend('/myapp/staging/')
    value = b.get_setting('DEBUG')

    assert value == 'off'
    assert fake_boto3.client.calls == [pretend.call('ssm')]
    assert fake_client.get_parameter.calls == [
        pretend.call(Name='/myapp/staging/DEBUG', WithDecryption=True),
    ]


def test_awsssm_missing(monkeypatch):
    error = ClientError({'Error': {'Code': 'ParameterNotFound'}}, 'get_parameter')
    fake_client = pretend.stub(
        get_parameter=pretend.raiser(error),
    )
    fake_boto3 = pretend.stub(
        client=lambda service: fake_client,
    )

    monkeypatch.setattr('configstore.backends.awsssm.boto3', fake_boto3)

    b = AwsSsmBackend()
    value = b.get_setting('/app1/TEMPLATE_DEBUG')

    assert value is None


def test_awsssm_missing_with_prefix(monkeypatch):
    error = ClientError({'Error': {'Code': 'ParameterNotFound'}}, 'get_parameter')
    fake_client = pretend.stub(
        get_parameter=pretend.raiser(error),
    )
    fake_boto3 = pretend.stub(
        client=lambda service: fake_client,
    )

    monkeypatch.setattr('configstore.backends.awsssm.boto3', fake_boto3)

    b = AwsSsmBackend('/app1/')
    value = b.get_setting('TEMPLATE_DEBUG')

    assert value is None


def test_awsssm_error(monkeypatch):
    error = ClientError({'Error': {'Code': 'SomethingBad'}}, 'get_parameter')
    fake_client = pretend.stub(
        get_parameter=pretend.raiser(error),
    )
    fake_boto3 = pretend.stub(
        client=lambda service: fake_client,
    )

    monkeypatch.setattr('configstore.backends.awsssm.boto3', fake_boto3)

    b = AwsSsmBackend('/app1/')
    with pytest.raises(ClientError):
        b.get_setting('TEMPLATE_DEBUG')



from .backends import AwsSsmBackend, DockerSecretBackend, DotenvBackend, EnvVarBackend
from .store import Store, SettingNotFoundException

__all__ = [
    'Store', 'SettingNotFoundException',
    'EnvVarBackend', 'DotenvBackend', 'DockerSecretBackend', 'AwsSsmBackend',
]



class SettingNotFoundException(Exception):
    pass


_no_default = object()


class Store(object):

    def __init__(self, backends):
        self._backends = tuple(backends)

    def add_backend(self, backend):
        self._backends += (backend,)

    def get_setting(self, key, default=_no_default):
        for backend in self._backends:
            ret = backend.get_setting(key)
            if ret is None:
                continue

            return ret

        if default is not _no_default:
            return default
        else:
            raise SettingNotFoundException(
                "Couldn't find setting {} in any backend".format(key))



from __future__ import absolute_import

try:
    import dotenv
except ImportError:  # pragma: no cover
    dotenv = None


class DotenvBackend(object):
    """Create an instance with a path to the .env file."""

    def __init__(self, dotenv_path):
        if dotenv is None:
            raise ImportError('install configstore[dotenv] to use '
                              'the dotenv backend')

        with open(dotenv_path) as file:
            content = file.read()

        self.config = dotenv.parse_dotenv(content)

    def get_setting(self, key):
        return self.config.get(key)



import configstore

store = configstore.Store([
    configstore.DockerSecretBackend(),
    configstore.EnvVarBackend(),
])

# Will raise an exception if APP_SECRET_KEY isn't found in any backend
SECRET_KEY = store.get_setting('APP_SECRET_KEY')

# Will use 'defaultvalue' if APP_SECRET_KEY isn't found
SECRET_KEY = store.get_setting('APP_SECRET_KEY', 'defaultvalue')

del store



from .awsssm import AwsSsmBackend
from .docker_secret import DockerSecretBackend
from .dotenv import DotenvBackend
from .env_var import EnvVarBackend

__all__ = [
    'EnvVarBackend', 'DotenvBackend', 'DockerSecretBackend', 'AwsSsmBackend',
]



from setuptools import setup

long_description = '''
configstore is a small pluggable library that lets you retrieve settings
or secrets from a variety of storage systems to configure your app.
'''

setup(
    name='configstore',
    version='0.4',
    description='Retrieve settings and secrets from different stores',
    long_description=long_description,
    url='https://github.com/caravancoop/configstore',
    author='Caravan Coop',
    author_email='hi@caravan.coop',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    packages=[
        'configstore', 'configstore.backends',
    ],
    extras_require={
        # django-dotenv does not actually use or need Django
        'dotenv': ['django-dotenv'],
        'awsssm': ['boto3'],
    },
)
